#include "bus.h"
#include "backend-vdcs.h"
#include "dss.h"
#include "model/apartment.h"
#include "model/dsmeter.h"

static ds::LogChannel dsLogChannel("bus");

namespace dss {

Bus::Bus(DSS& dss, ModelMaintenance& modelMaintenance, Security& security)
    : m_dss(dss),
      m_ds485Bus(&dss, &modelMaintenance, security),
      m_ds485Query(*m_ds485Bus.getStructureQueryBusInterface()),
      m_ds485Modify(*m_ds485Bus.getStructureModifyingBusInterface()),
      m_queries{std::ref(m_ds485Query)},
      m_modifies{std::ref(m_ds485Modify)} {}

StructureQueryBusInterface& Bus::dsmQueryBus(const dsuid_t& dsmId) {
  if (dsmId == DSUID_BROADCAST) {
    return m_ds485Query;
  }
  const auto& type = m_dss.getApartment().getDSMeterByDSID(dsmId)->getBusMemberType();
  if (type == BusMember_backendVdc) {
    return m_dss.backendVdcs().structureQueryBusInterface();
  }
  return m_ds485Query;
}

StructureModifyingBusInterface& Bus::dsmModifyBus(const dsuid_t& dsmId) {
  if (dsmId == DSUID_BROADCAST) {
    return m_ds485Modify;
  }
  const auto& type = m_dss.getApartment().getDSMeterByDSID(dsmId)->getBusMemberType();
  if (type == BusMember_backendVdc) {
    return m_dss.backendVdcs().structureModifyingBusInterface();
  }
  return m_ds485Modify;
}

std::string Bus::name() const { return dsLogChannel.name(); }

void Bus::removeDeviceFromDSMeters(const dsuid_t& device) {
  for (const auto& modify : m_modifies) {
    try {
      modify.get().removeDeviceFromDSMeters(device);
    } catch (const std::exception& e) {
      DS_ERROR("Failed to remove device.", device, modify);
    }
  }
}

void Bus::sceneSetName(uint16_t zone, uint8_t group, uint8_t scene, const std::string& name) {
  for (const auto& modify : m_modifies) {
    try {
      modify.get().sceneSetName(zone, group, scene, name);
    } catch (const std::exception& e) {
      DS_ERROR("Failed to set scene name.", zone, scene, name, modify);
    }
  }
}

} // namespace dss
