/*
    Copyright (c) 2009,2011 digitalSTROM.org, Zurich, Switzerland

    Author: Patrick Staehlin, futureLAB AG <pstaehlin@futurelab.ch>

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/
#pragma once

#include <trompeloeil.hpp>
#include "businterface.h"
#include "vdc-messages.pb.h"

namespace dss {

class StructureQueryBusInterfaceMock : public trompeloeil::mock_interface<StructureQueryBusInterface> {
public:
  IMPLEMENT_MOCK0(getBusMembers);
  IMPLEMENT_MOCK1(getDSMeterSpec);
  IMPLEMENT_MOCK1(getZones);
  IMPLEMENT_MOCK3(getDevicesInZone);
  IMPLEMENT_MOCK2(getInactiveDevicesInZone);
  IMPLEMENT_MOCK2(getGroups);
  IMPLEMENT_MOCK1(getClusters);
  IMPLEMENT_MOCK1(getPowerStates);
  IMPLEMENT_MOCK1(getMeteringConfiguration);
  IMPLEMENT_MOCK2(getLastCalledScenes);
  IMPLEMENT_MOCK2(getZoneStates);
  IMPLEMENT_MOCK3(getEnergyBorder);
  IMPLEMENT_MOCK2(deviceGetSpec);
  IMPLEMENT_MOCK3(getSceneName);
  IMPLEMENT_MOCK1(getDSMeterHash);
  IMPLEMENT_MOCK2(getDSMeterState);
  IMPLEMENT_MOCK2(getZoneHeatingConfig);
  IMPLEMENT_MOCK2(getZoneHeatingInternals);
  IMPLEMENT_MOCK2(getZoneHeatingState);
  IMPLEMENT_MOCK2(getZoneHeatingOperationModes);
  IMPLEMENT_MOCK3(getZoneSensor);
  IMPLEMENT_MOCK5(getZoneSensorValue);
  IMPLEMENT_MOCK3(getDevicesCountInZone);
  IMPLEMENT_MOCK5(protobufMessageRequest);
  IMPLEMENT_MOCK3(getVdcProperty);
};

class StructureModifyingBusInterfaceMock : public trompeloeil::mock_interface<StructureModifyingBusInterface> {
public:
  IMPLEMENT_CONST_MOCK0(name);
  IMPLEMENT_MOCK3(setZoneID);
  IMPLEMENT_MOCK2(createZone);
  IMPLEMENT_MOCK2(removeZone);
  IMPLEMENT_MOCK3(addToGroup);
  IMPLEMENT_MOCK3(removeFromGroup);
  IMPLEMENT_MOCK2(removeDeviceFromDSMeter);
  IMPLEMENT_MOCK1(removeDeviceFromDSMeters);
  IMPLEMENT_MOCK4(sceneSetName);
  IMPLEMENT_MOCK3(deviceSetName);
  IMPLEMENT_MOCK2(meterSetName);
  IMPLEMENT_MOCK5(createGroup);
  IMPLEMENT_MOCK2(removeGroup);
  IMPLEMENT_MOCK4(groupSetApplication);
  IMPLEMENT_MOCK3(groupSetName);
  IMPLEMENT_MOCK4(createCluster);
  IMPLEMENT_MOCK1(removeCluster);
  IMPLEMENT_MOCK2(clusterSetName);
  IMPLEMENT_MOCK3(clusterSetApplication);
  IMPLEMENT_MOCK4(clusterSetProperties);
  IMPLEMENT_MOCK2(clusterSetLockedScenes);
  IMPLEMENT_MOCK2(clusterSetConfigurationLock);
  IMPLEMENT_MOCK2(clusterSetOperationLock);
  IMPLEMENT_MOCK3(setButtonSetsLocalPriority);
  IMPLEMENT_MOCK3(setButtonCallsPresent);
  IMPLEMENT_MOCK3(setZoneHeatingConfig);
  IMPLEMENT_MOCK3(setZoneHeatingOperationModes);
  IMPLEMENT_MOCK3(setZoneSensor);
  IMPLEMENT_MOCK2(resetZoneSensor);
  IMPLEMENT_MOCK4(setCircuitPowerStateConfig);
  IMPLEMENT_MOCK2(setMeteringConfiguration);
  IMPLEMENT_MOCK2(setProperty);
  IMPLEMENT_MOCK2(getProperty);
  IMPLEMENT_MOCK2(setFireMuteEnabled);
  IMPLEMENT_MOCK2(removeVdc);
  IMPLEMENT_MOCK2(clusterSetStopTimeout);
  IMPLEMENT_MOCK3(zoneSetStopTimeout);
};

class DeviceBusInterfaceMock : public trompeloeil::mock_interface<DeviceBusInterface> {
public:
  IMPLEMENT_MOCK3(getDeviceConfig);
  IMPLEMENT_MOCK3(getDeviceConfigWord);
  IMPLEMENT_MOCK4(setDeviceConfig);
  IMPLEMENT_MOCK2(setDeviceButtonActiveGroup);
  IMPLEMENT_MOCK2(setDeviceProgMode);
  IMPLEMENT_MOCK2(increaseDeviceOutputChannelValue);
  IMPLEMENT_MOCK2(decreaseDeviceOutputChannelValue);
  IMPLEMENT_MOCK2(stopDeviceOutputChannelValue);
  IMPLEMENT_MOCK2(getDeviceOutputChannelValue);
  IMPLEMENT_MOCK5(setDeviceOutputChannelValue);
  IMPLEMENT_MOCK3(getDeviceOutputChannelSceneValue);
  IMPLEMENT_MOCK5(setDeviceOutputChannelSceneValue);
  IMPLEMENT_MOCK2(getDeviceOutputChannelSceneConfig);
  IMPLEMENT_MOCK3(setDeviceOutputChannelSceneConfig);
  IMPLEMENT_MOCK3(setDeviceOutputChannelDontCareFlags);
  IMPLEMENT_MOCK2(getDeviceOutputChannelDontCareFlags);
  IMPLEMENT_MOCK1(getTransmissionQuality);
  IMPLEMENT_MOCK2(setValue);
  IMPLEMENT_MOCK2(getSensorValue);
  IMPLEMENT_MOCK2(getSensorValueEx);
  IMPLEMENT_MOCK2(addGroup);
  IMPLEMENT_MOCK2(removeGroup);
  IMPLEMENT_MOCK2(lockOrUnlockDevice);
  IMPLEMENT_MOCK4(genericRequest);
  IMPLEMENT_MOCK2(setProperty);
  IMPLEMENT_MOCK2(getProperty);
  IMPLEMENT_MOCK1(reregister);
};

class ActionRequestInterfaceMock : public trompeloeil::mock_interface<ActionRequestInterface> {
public:
  IMPLEMENT_MOCK6(callScene);
  IMPLEMENT_MOCK5(callSceneMin);
  IMPLEMENT_MOCK4(saveScene);
  IMPLEMENT_MOCK5(undoScene);
  IMPLEMENT_MOCK4(undoSceneLast);
  IMPLEMENT_MOCK4(blink);
  IMPLEMENT_MOCK5(setValue);
  IMPLEMENT_MOCK5(increaseOutputChannelValue);
  IMPLEMENT_MOCK5(decreaseOutputChannelValue);
  IMPLEMENT_MOCK5(stopOutputChannelValue);
  IMPLEMENT_MOCK7(pushSensor);
  IMPLEMENT_MOCK0(muteFire);
  MAKE_MOCK6(setState, void(Group*, const uint8_t, const uint8_t, const uint8_t, const uint8_t, const uint8_t),
             override);
  MAKE_MOCK2(setState, void(Cluster*, const uint8_t), override);
  MAKE_MOCK2(stopStopTimer, void(Group*, const uint8_t), override);
  MAKE_MOCK1(stopStopTimer, void(Cluster*), override);
  IMPLEMENT_MOCK2(isOperationLock);
};

class BusInterfaceMock final : public trompeloeil::mock_interface<BusInterface> {
public:
  DeviceBusInterface* getDeviceBusInterface() override { return &device; }
  StructureQueryBusInterface* getStructureQueryBusInterface() override { return &structureQuery; }
  MeteringBusInterface* getMeteringBusInterface() override { DS_FAIL_REQUIRE("not implemented"); }
  StructureModifyingBusInterface* getStructureModifyingBusInterface() override { return &structureModify; }
  ActionRequestInterface* getActionRequestInterface() override { return &action; }

  IMPLEMENT_MOCK1(setBusEventSink);
  IMPLEMENT_MOCK0(getConnectionURI);
  IMPLEMENT_MOCK1(add);

  DeviceBusInterfaceMock device;
  ActionRequestInterfaceMock action;
  StructureQueryBusInterfaceMock structureQuery;
  StructureModifyingBusInterfaceMock structureModify;
};

} // namespace dss
