/*
 *  Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of digitalSTROM Server.
 *
 *  digitalSTROM Server is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  digitalSTROM Server is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
 */

#include "digest.h"
#include <ds/log.h>
#include <openssl/sha.h>
#include <stdio.h>

namespace dsCrypt {

static_assert(Sha256{}.size() == SHA256_DIGEST_LENGTH);

Sha256 sha256Sum(gsl::span<const gsl::byte> input) {
  auto result = Sha256{};

  SHA256_CTX context;
  if (!SHA256_Init(&context)) {
    DS_FAIL_REQUIRE("sha256 failed");
  }

  if (!SHA256_Update(&context, reinterpret_cast<const unsigned char*>(input.data()), input.size())) {
    DS_FAIL_REQUIRE("sha256 failed");
  }

  if (!SHA256_Final(reinterpret_cast<unsigned char*>(result.data()), &context)) {
    DS_FAIL_REQUIRE("sha256 failed");
  }

  return result;
}

} // namespace dsCrypt
