/*
    Copyright (c) 2020 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "session.h"

#include <ds/asio/test.h>
#include <ds/catch/catch.h>
#include <ds/log.h>

using namespace trompeloeil;

namespace {

static const char *TAGS = "[dsNetSession][dsNet]";

struct SessionMock : ds::asio::Resource<>::Resource, trompeloeil::mock_interface<dsNet::ISession> {
  using ds::asio::Resource<>::Resource;
  IMPLEMENT_MOCK0(close);
  MAKE_MOCK0(asyncRun, void());
  MAKE_MOCK0(sharedSession, std::shared_ptr<ISession>());
};

SCENARIO("dsNetClosePtr", TAGS) {
  auto loop = ds::asio::TestLoopResource{};
  GIVEN("subclassed session") {
    auto session = std::make_shared<SessionMock>(loop, "session");
    GIVEN("ClosePtr<Session>") {
      auto close = NAMED_REQUIRE_CALL(*session, close());
      auto closePtr = dsNet::ClosePtr<dsNet::ISession>{std::shared_ptr{session}};
      CHECK(static_cast<bool>(closePtr));
      SECTION("reset calls close immediately and does not call it later") {
        auto close2 = std::move(close);
        closePtr.reset();
        CHECK(!static_cast<bool>(closePtr));
      }
      SECTION("session->call does not inflence close from closePtr") {
        REQUIRE_CALL(*session, close());
        session->close();
      }
    }
  }
  GIVEN("ClosePtr<SessionMock>") {
    auto close = std::unique_ptr<trompeloeil::expectation>{}; // close expectation must overlive closePtr
    auto closePtr = dsNet::ClosePtr{std::make_shared<SessionMock>(loop, "session")};
    close = NAMED_REQUIRE_CALL(*closePtr, close());
    SECTION("session method can be called") {
      REQUIRE_CALL(*closePtr, asyncRun());
      closePtr->asyncRun();
    }
  }
}

} // namespace
