/*
    Copyright (c) 2020 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "tcp-acceptor.h"

#include <ds/asio/test.h>
#include <ds/log.h>

#include <boost/asio/connect.hpp>

using namespace std::literals;
using namespace trompeloeil;
using boost::system::error_code;
using tcp = boost::asio::ip::tcp; // from <boost/asio/ip/tcp.hpp>

namespace {

static const char* TAGS = "[dsNetTcpAcceptor]";

struct Accepted {
public:
  MAKE_MOCK1(call, void(const tcp::socket&));
};

SCENARIO("dsNetTcpAcceptor", TAGS) {
  auto loop = ds::asio::TestLoopResource{};
  auto accepted = Accepted{};
  auto acceptor = dsNet::makeTcpAcceptor(loop, {.endpoint = {.address = "127.0.0.1"}});
  acceptor->asyncRun({.accept = [&](tcp::socket&& x) { accepted.call(std::move(x)); }});
  auto resolver = tcp::resolver{loop.io};
  auto socket = tcp::socket{loop.io};
  const auto endpoints = resolver.resolve(acceptor->localAsioEndpoint());
  SECTION("client connects") {
    boost::asio::async_connect(socket, endpoints, [&](error_code ec, auto&&) {
      CHECK(!ec);
      loop.stop();
    });
    REQUIRE_CALL(accepted, call(_))
        .LR_SIDE_EFFECT(CHECK(_1.local_endpoint().port() == acceptor->localEndpoint().port); loop.stop());
    loop.run();
    loop.run();
  }

  SECTION("acceptor destroyed -> connection fails") {
    acceptor.reset();
    boost::asio::async_connect(socket, endpoints, [&](error_code ec, auto&&) {
      CHECK(ec);
      loop.stop();
    });
    loop.run();
  }

  SECTION("localEndpoint, localAsioEndpoint") {
    CHECK(acceptor->localEndpoint().toAsio() == acceptor->localAsioEndpoint());
  }
}

} // namespace
