/*
    Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#include "loop-impl.h"

#include <ds/log.h>
#include <sys/prctl.h>

static ds::LogChannel dsLogChannel("dsAsioLoopImpl");

namespace ds::asio {

LoopImpl::~LoopImpl() {
  DS_DEBUG_ENTER(name);
  io.stop();
  thread.join();
}

void LoopImpl::run() {
  DS_DEBUG_ENTER(name);
  prctl(PR_SET_NAME, reinterpret_cast<unsigned long>(name.data()), 0, 0, 0);
  for (;;) {
    try {
      io.run();
      break; // run() exited normally
    } catch (std::exception &e) {
      DS_ERROR("Unhandled exception", name, e);
    }
  }
}

} // namespace ds::asio
