#include "date-time.h"
#include <ds/log.h>
#include <gsl.h>
#include <stdio.h>
#include <sys/time.h>
#include <boost/algorithm/string.hpp>

namespace ds {

std::string toLocalMsIsoString(DateTime timePoint) {
  struct tm tm;
  char buf[sizeof "2011-10-08T07:07:09.000+02:00"];
  char fmt[sizeof "2011-10-08T07:07:09.000+02:00"];

  // http://stackoverflow.com/questions/1551597/using-strftime-in-c-how-can-i-format-time-exactly-like-a-unix-timestamp
  auto count = std::chrono::duration_cast<std::chrono::milliseconds>(timePoint.time_since_epoch()).count();
  auto time = gsl::narrow_cast<time_t>(count / 1000);
  auto msec = gsl::narrow_cast<int>(count % 1000);
  ::localtime_r(&time, &tm);
  ::strftime(fmt, sizeof fmt, "%FT%T.%%03d%z", &tm);
  ::snprintf(buf, sizeof buf, fmt, msec);

  /* ensure there is a colon in time zone */
  if (buf[26] != ':') {
    buf[29] = '\0';
    buf[28] = buf[27];
    buf[27] = buf[26];
    buf[26] = ':';
  }
  return std::string(buf);
}

std::string toString(DateTime timePoint) {
  struct tm tm;
  char buf[sizeof "2009-01-02T09:00:00.000Z"];
  char fmt[sizeof "2009-01-02T09:00:00.%03uZ"];

  // http://stackoverflow.com/questions/361363/how-to-measure-time-in-milliseconds-using-ansi-c
  // http://stackoverflow.com/questions/1551597/using-strftime-in-c-how-can-i-format-time-exactly-like-a-unix-timestamp
  auto count = std::chrono::duration_cast<std::chrono::milliseconds>(timePoint.time_since_epoch()).count();
  auto time = gsl::narrow_cast<time_t>(count / 1000);
  auto msec = gsl::narrow_cast<int>(count % 1000);
  ::gmtime_r(&time, &tm);
  ::strftime(fmt, sizeof fmt, "%FT%T.%%03dZ", &tm);
  ::snprintf(buf, sizeof buf, fmt, msec);
  return std::string(buf);
}

DateTime toDateTime(gsl::cstring_span s) {
  struct tm tm {};

  DS_REQUIRE(s.size() >= 20, "Expecting DateTime in format 2009-01-02T09:00:00[.000]Z", s, s.size());
  DS_REQUIRE(boost::ends_with(s, "Z"), "Expecting DateTime in format 2009-01-02T09:00:00[.000]Z", s);

  auto parsedTokens = ::sscanf(s.data(), "%04d-%02d-%02dT%02d:%02d:%02d", &tm.tm_year, &tm.tm_mon, &tm.tm_mday,
                               &tm.tm_hour, &tm.tm_min, &tm.tm_sec);
  DS_REQUIRE(parsedTokens == 6, "Expecting DateTime in format 2009-01-02T09:00:00[.000]Z", s);
  DS_REQUIRE(tm.tm_year >= 1900, "Expecting DateTime after year 1900", s);
  tm.tm_year -= 1900;
  DS_REQUIRE(tm.tm_mon >= 1, "Expecting DateTime month >= 1", s);
  tm.tm_mon -= 1;

  // parse optional milliseconds
  auto ms = int{};
  if (s[19] == '.') {
    if (isdigit(s[20])) {
      ms += (s[20] - '0') * 100;
      if (isdigit(s[21])) {
        ms += (s[21] - '0') * 10;
        if (isdigit(s[22])) {
          ms += (s[22] - '0') * 1;
        }
      }
    }
  }
  return std::chrono::system_clock::from_time_t(::timegm(&tm)) + std::chrono::milliseconds(ms);
}

} // namespace ds
