/*
    Copyright (c) 2010 digitalSTROM.org, Zurich, Switzerland

    Authors: Patrick Staehlin, futureLAB AG <pstaehlin@futurelab.ch>
             Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef DSDEVICEBUSINTERFACE_H_
#define DSDEVICEBUSINTERFACE_H_

#include <memory>

#include "businterface.h"
#include "ds485/dsbusinterfaceobj.h"
#include "taskprocessor.h"

using std::shared_ptr;

namespace dss {
class Device;
class DSMeter;
class ModelMaintenance;

class DSDeviceBusInterface : public DSBusInterfaceObj, public DeviceBusInterface {
public:
  class ConfigReaderHelper {
  public:
    ConfigReaderHelper(const std::string& _busConnection);
    virtual ~ConfigReaderHelper();

    virtual void setup(std::shared_ptr<Device> _device);

    uint16_t getDeviceConfigWord(uint8_t _configClass, uint8_t _configIndex) const;
    uint8_t getDeviceConfig(uint8_t _configClass, uint8_t _configIndex) const;

  protected:
    std::string m_busConnection;
    DsmApiHandle_t m_dsmApiHandle;
    devid_t m_deviceAdress;
    dsuid_t m_dsmId;
    std::shared_ptr<DSMeter> m_dsm;
    int m_revisionID;
  };
  class OEMDataReader : public Task, public ConfigReaderHelper, public std::enable_shared_from_this<OEMDataReader> {
  public:
    OEMDataReader(const std::string& _busConnection);
    virtual ~OEMDataReader();
    virtual void run();
  };

  class TNYConfigReader : public Task,
                          public ConfigReaderHelper,
                          public std::enable_shared_from_this<TNYConfigReader> {
  public:
    TNYConfigReader(const std::string& _busConnection);
    virtual ~TNYConfigReader();
    virtual void run();
  };

  DSDeviceBusInterface() : DSBusInterfaceObj(), m_pModelMaintenance(NULL) {}

  virtual uint8_t getDeviceConfig(const Device& _device, uint8_t _configClass, uint8_t _configIndex) override;

  virtual uint16_t getDeviceConfigWord(const Device& _device, uint8_t _configClass, uint8_t _configIndex) override;

  virtual void setDeviceConfig(const Device& _device, uint8_t _configClass, uint8_t _configIndex,
                               uint8_t _value) override;

  virtual void setDeviceButtonActiveGroup(const Device& _device, uint8_t _groupID) override;

  virtual void setDeviceProgMode(const Device& _device, uint8_t _modeId) override;

  virtual void setValue(const Device& _device, uint8_t _value) override;

  virtual void addGroup(const Device& _device, const int _groupId) override;
  virtual void removeGroup(const Device& _device, const int _groupId) override;

  virtual uint16_t getSensorValue(const Device& _device, const int _sensorIndex) override;
  virtual DeviceSensorValue_t getSensorValueEx(const Device& _device, const int _sensorIndex) override;

  virtual void lockOrUnlockDevice(const Device& _device, const bool _lock) override;

  virtual std::pair<uint8_t, uint16_t> getTransmissionQuality(const Device& _device) override;

  virtual void increaseDeviceOutputChannelValue(const Device& _device, uint8_t _channel) override;

  virtual void decreaseDeviceOutputChannelValue(const Device& _device, uint8_t _channel) override;

  virtual void stopDeviceOutputChannelValue(const Device& _device, uint8_t _channel) override;

  virtual uint16_t getDeviceOutputChannelValue(const Device& _device, uint8_t _channel) override;

  virtual void setDeviceOutputChannelValue(const Device& _device, uint8_t _channel, uint8_t _size, uint16_t _value,
                                           bool _applyNow = true) override;

  virtual uint16_t getDeviceOutputChannelSceneValue(const Device& _device, uint8_t _channel, uint8_t _scene) override;

  virtual void setDeviceOutputChannelSceneValue(const Device& _device, uint8_t _channel, uint8_t _size, uint8_t _scene,
                                                uint16_t _value) override;

  virtual uint16_t getDeviceOutputChannelSceneConfig(const Device& _device, uint8_t _scene) override;

  virtual void setDeviceOutputChannelSceneConfig(const Device& _device, uint8_t _scene, uint16_t _value) override;
  virtual void setDeviceOutputChannelDontCareFlags(const Device& _device, uint8_t _scene, uint16_t _value) override;
  virtual uint16_t getDeviceOutputChannelDontCareFlags(const Device& _device, uint8_t _scene) override;

  virtual void genericRequest(const Device& _device, const std::string& methodName,
                              const ::google::protobuf::RepeatedPtrField< ::vdcapi::PropertyElement>& params,
                              const std::locale&) override;

  virtual void setProperty(const Device& _device,
                           const ::google::protobuf::RepeatedPtrField< ::vdcapi::PropertyElement>& properties) override;
  virtual vdcapi::Message getProperty(
      const Device& _device, const ::google::protobuf::RepeatedPtrField< ::vdcapi::PropertyElement>& query) override;

  virtual void reregister(const Device& _device) override;

  void setModelMaintenace(ModelMaintenance* _modelMaintenance) { m_pModelMaintenance = _modelMaintenance; }

private:
  ModelMaintenance* m_pModelMaintenance;

}; // DSDeviceBusInterface

} // namespace dss

#endif
