/*
    Copyright (c) 2018 digitalSTROM.org," " Zurich," " Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation," " either version 3 of the License," " or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful," "
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not," " see <http://www.gnu.org/licenses/>.
*/

#include <ds/catch/catch.h>
#include <ds/log.h>

#include "dsm-api-mock.h"

#include "ds485-query.h"

static ds::LogChannel dsLogChannel("ds485queryTest");
static const char* TAGS = "[dss][ds485query]";

using namespace dss;
using namespace trompeloeil;
using trompeloeil::_;

namespace {

TEST_CASE("getDevicesInZoneWrongDeviceCount", TAGS) {
  DSStructureQueryBusInterface query;
  int dummyHandle = 42;
  DsmApiHandle_t handle = &dummyHandle;
  query.setDSMApiHandle(handle);
  Dsuid dsm{"0000000000000000000000000000000001"};

  REQUIRE_CALL(dsm_api_mock, ZoneDeviceCount_all(handle, dsm, 1, _)).SIDE_EFFECT(*_4 = 1).RETURN(0);
  REQUIRE_CALL(dsm_api_mock, DeviceInfo_by_index(handle, dsm, 1, 0, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _))
      .RETURN(0);

  REQUIRE_NOTHROW([&]() {
    auto devices = query.getDevicesInZone(dsm, 1, false);
    REQUIRE(devices.size() == 1);
  }());

  REQUIRE_CALL(dsm_api_mock, ZoneDeviceCount_all(handle, dsm, 1, _)).SIDE_EFFECT(*_4 = 2).RETURN(0);
  REQUIRE_CALL(dsm_api_mock, DeviceInfo_by_index(handle, dsm, 1, 0, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _))
      .RETURN(0);
  REQUIRE_CALL(dsm_api_mock, DeviceInfo_by_index(handle, dsm, 1, 1, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _))
      .RETURN(3);

  REQUIRE_NOTHROW([&]() {
    auto devices = query.getDevicesInZone(dsm, 1, false);
    REQUIRE(devices.size() == 1);
  }());

  REQUIRE_CALL(dsm_api_mock, ZoneDeviceCount_all(handle, dsm, 1, _)).SIDE_EFFECT(*_4 = 1).RETURN(0);
  REQUIRE_CALL(dsm_api_mock, DeviceInfo_by_index(handle, dsm, 1, 0, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _))
      .RETURN(4);

  REQUIRE_THROWS(query.getDevicesInZone(dsm, 1, false));
}

} // namespace
