/*
    Copyright (c) 2010 digitalSTROM.org, Zurich, Switzerland

    Author: Patrick Staehlin, futureLAB AG <pstaehlin@futurelab.ch>

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#pragma once

#include "businterface.h"
#include "dsbusinterfaceobj.h"

namespace dss {

class DSActionRequest : public DSBusInterfaceObj, public ActionRequestInterface {
public:
  DSActionRequest() : DSBusInterfaceObj(), m_pBusEventSink(NULL) {}

  void callScene(AddressableModelItem *pTarget, const CallOrigin origin, const SceneAccessCategory category,
                 const uint16_t _scene, const std::string token, const bool _force) override;
  void callSceneMin(AddressableModelItem *pTarget, const CallOrigin origin, const SceneAccessCategory category,
                    const uint16_t _scene, const std::string token) override;
  void saveScene(AddressableModelItem *pTarget, const CallOrigin origin, const uint16_t _scene,
                 const std::string token) override;
  void undoScene(AddressableModelItem *pTarget, const CallOrigin origin, const SceneAccessCategory category,
                 const uint16_t _scene, const std::string token) override;
  void undoSceneLast(AddressableModelItem *pTarget, const CallOrigin origin, const SceneAccessCategory category,
                     const std::string token) override;
  void blink(AddressableModelItem *pTarget, const CallOrigin origin, const SceneAccessCategory category,
             const std::string token) override;
  void setValue(AddressableModelItem *pTarget, const CallOrigin origin, const SceneAccessCategory category,
                const uint8_t value, const std::string token) override;
  void increaseOutputChannelValue(AddressableModelItem *pTarget, const CallOrigin origin,
                                  const SceneAccessCategory category, ChannelType channel,
                                  const std::string token) override;
  void decreaseOutputChannelValue(AddressableModelItem *pTarget, const CallOrigin origin,
                                  const SceneAccessCategory category, ChannelType channel,
                                  const std::string token) override;
  void stopOutputChannelValue(AddressableModelItem *pTarget, const CallOrigin origin,
                              const SceneAccessCategory category, ChannelType channel,
                              const std::string token) override;
  void pushSensor(AddressableModelItem *pTarget, const CallOrigin origin, const SceneAccessCategory category,
                  dsuid_t _sourceID, SensorType _sensorType, double _sensorValueFloat,
                  const std::string token) override;
  void muteFire() override;
  void setState(Group *group, const uint8_t zoneState, const uint8_t areaState1, const uint8_t areaState2,
                const uint8_t areaState3, const uint8_t areaState4) override;
  void setState(Cluster *cluster, const uint8_t state) override;
  void stopStopTimer(Group *group, const uint8_t zoneOrArea) override;
  void stopStopTimer(Cluster *cluster) override;

  void setBusEventSink(BusEventSink *eventSink);

  //< @ret true if locked
  bool isOperationLock(const dsuid_t &dSM, int clusterId) override;

private:
  BusEventSink *m_pBusEventSink;
}; // DSActionRequest

} // namespace dss
