/*
 *  Copyright (c) 2019 digitalSTROM AG, Zurich, Switzerland
 *
 *  This file is part of digitalSTROM Server.
 *
 *  digitalSTROM Server is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  digitalSTROM Server is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <trompeloeil.hpp>
#include "hal/hardware-abstraction.h"

using namespace dss::hal;

namespace dss {

class HardwareAbstractionMock : public trompeloeil::mock_interface<HardwareAbstraction> {
public:
  MAKE_CONST_MOCK3(write, void(const Dsuid&, const std::string&, double));
  MAKE_CONST_MOCK3(write, void(const Dsuid&, const std::string&, bool));
  MAKE_CONST_MOCK3(write, void(const Dsuid&, const std::string&, int));
  MAKE_CONST_MOCK3(write, void(const Dsuid&, const std::string&, const std::string&));
  IMPLEMENT_CONST_MOCK4(updateDeviceConfig);

  MAKE_CONST_MOCK2(readDouble, double(const Dsuid& dsuid, const std::string& parameter), override);
  MAKE_CONST_MOCK1(readBool, bool(const DevicePropertyId& id), override);
  MAKE_CONST_MOCK1(readBool, bool(const DeviceScenePropertyId& id), override);
  MAKE_CONST_MOCK1(readBool, bool(const DeviceSceneChannelPropertyId& id), override);
  MAKE_CONST_MOCK2(readString, std::string(const Dsuid& dsuid, const std::string& parameter), override);

  MAKE_CONST_MOCK4(callScene, bool(const Dsuid& dsuid, int sceneId, bool forced, ds::DateTime now), override);
  MAKE_CONST_MOCK3(undoScene, bool(const Dsuid& dsuid, int sceneId, ds::DateTime now), override);
  IMPLEMENT_CONST_MOCK3(saveScene);
  MAKE_CONST_MOCK3(setOutputValue, bool(const Dsuid& dsuid, uint8_t value, ds::DateTime now), override);
  MAKE_CONST_MOCK4(setOutputValue, bool(const Dsuid& dsuid, ChannelType channel, double value, ds::DateTime now),
                   override);
  MAKE_CONST_MOCK2(setSceneLocalPrio, bool(const Dsuid& dsuid, int sceneId), override);
  MAKE_CONST_MOCK3(callSceneMin, bool(const Dsuid& dsuid, int sceneId, ds::DateTime now), override);
  MAKE_CONST_MOCK2(setOperationLock, void(const Dsuid& device, bool lock), override);
  IMPLEMENT_CONST_MOCK3(stopDimming);
  IMPLEMENT_CONST_MOCK2(completeMoving);

  IMPLEMENT_CONST_MOCK1(getState);
  MAKE_CONST_MOCK1(getZoneState, dsModel::ZoneApplicationsStatus(int zoneId), override);

  MAKE_CONST_MOCK1(cacheStatus, DeviceCacheState(const Dsuid& dsuid), override);
  IMPLEMENT_CONST_MOCK1(populateCache);

  IMPLEMENT_CONST_MOCK1(deviceCacheable);
  IMPLEMENT_CONST_MOCK0(getCache);
  IMPLEMENT_CONST_MOCK0(getStateCache);

  MAKE_CONST_MOCK1(areaMember, std::vector<int>(const Dsuid& dsuid), override);

  IMPLEMENT_CONST_MOCK1(getDeviceThresholds);
  MAKE_CONST_MOCK1(getPlayerScenarios, std::vector<int>(int zoneId), override);

  IMPLEMENT_CONST_MOCK1(publishToPropertyTree);
};

} // namespace dss
