/*
    Copyright (c) 2009,2010,2011 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#pragma once

#include <digitalSTROM/dsuid.h>
#include <ds/date-time.h>
#include <ds/reflect.h>
#include <ds/system-fwd.h>
#include <ds485types.h>

#include <vector>

#include "datetools.h"
#include "metering/metering-fwd.h"

namespace dss {
class RRDInterface;
struct MeteringConfig;
struct ArchiveValue;
struct ValueRange;
}

namespace dss::metering {

using ArchiveId = std::pair<Dsuid, Category>;

class Archives {
public:
  virtual ~Archives() = default;
  virtual const std::vector<MeteringConfig> getConfig() const = 0;

  virtual std::string filename(const ArchiveId& archiveId) const = 0;
  virtual void create(const ArchiveId& archiveId) = 0;
  virtual bool hasArchive(const ArchiveId& archiveId) const = 0;

  virtual void addValue(const ArchiveId& archiveId, ArchiveValue&& value) = 0;
  virtual std::optional<ArchiveValue> lastValue(const ArchiveId& archiveId) = 0;

  virtual void flush(std::vector<ArchiveId> archiveIds) const = 0;
  virtual ValueRange xport(time_t start, time_t end, int step, std::vector<ArchiveId> archiveIds, SeriesTypes type,
                           bool energyInWh) = 0;

  // unit tests
  virtual bool validateDBSeries(const ArchiveId& archiveId, bool& _tunePowerMaxSetting) = 0;
  virtual void tuneDBPowerSettings(const ArchiveId& archiveId) = 0;
  virtual bool withoutUpdates(const ArchiveId& archiveId) const = 0;
  virtual bool checkDBFlush(const DateTime& _sampledAt, const ArchiveId& archiveId) const = 0;
  virtual void restartCached() const = 0;

  virtual void setRRDCachedPath(std::string rrdCachedPath) = 0;
  virtual void setMeteringStorageLocation(std::string&& meteringStorageLocation) = 0;
};

std::shared_ptr<Archives> makeArchives(std::shared_ptr<RRDInterface> rrdInterface, ds::System& system);

} // namespace dss::metering
