/*
    Copyright (c) 2009,2011 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#include "model/device-mock.h"

#include <boost/container/flat_map.hpp>

#include "businterface.h"
#include "busscanner.h" // setupBaseDevice
#include "dsm-api-mock.h"
#include "model/apartment.h"
#include "model/device.h"

static ds::LogChannel dsLogChannel("deviceMock");

using trompeloeil::_;

namespace dss {

struct PartInfo {
  uint16_t productId;
  int vendorId;
  uint8_t color;
};

auto toPartInfo = [](const HardwareType& hardwareType) {
  // Generated form productlist_dsd.csv -> scripts/extract-productlist-dsd.py
  static const boost::container::flat_map<HardwareType, PartInfo> partInfoTable = {
      {HardwareType::GE_KM200, {0x00C8, 1, 1}},    {HardwareType::GE_TKM210, {0x04D2, 1, 1}},
      {HardwareType::GE_SDM200, {0x08C8, 1, 1}},   {HardwareType::GE_SDM200JS, {0x08C9, 2, 1}},
      {HardwareType::GE_SDM200CS, {0x08CA, 2, 1}}, {HardwareType::GE_SDS200CW, {0x18C8, 2, 1}},
      {HardwareType::GE_SDS200CS, {0x18C9, 2, 1}}, {HardwareType::GE_SDS220CT, {0x18DC, 2, 1}},
      {HardwareType::GE_TKM220, {0x04DC, 1, 1}},   {HardwareType::GE_TKM230, {0x04E6, 1, 1}},
      {HardwareType::GE_KL200, {0x0CC8, 1, 1}},    {HardwareType::GE_UMV200, {0x2CC8, 2, 1}},
      {HardwareType::GE_UMV210, {0x2CD2, 2, 1}},   {HardwareType::GE_KM300, {0x012C, 2, 1}},
      {HardwareType::GE_TKM300, {0x052C, 2, 1}},   {HardwareType::GE_SDM300W, {0x092C, 2, 1}},
      {HardwareType::GE_SDM300S, {0x092D, 2, 1}},  {HardwareType::GE_ZWD300W, {0x3D2C, 2, 1}},
      {HardwareType::GN_KM200, {0x00C8, 1, 7}},    {HardwareType::GN_TKM200, {0x04C8, 1, 7}},
      {HardwareType::GN_TKM210, {0x04D2, 1, 7}},   {HardwareType::GR_TKM200, {0x04C8, 1, 2}},
      {HardwareType::GR_TKM210, {0x04D2, 1, 2}},   {HardwareType::GR_KL200, {0x0CC8, 1, 2}},
      {HardwareType::GR_KL210, {0x0CD2, 1, 2}},    {HardwareType::GR_KL220, {0x0CDC, 1, 2}},
      {HardwareType::GR_HKL230, {0x0CE6, 2, 2}},   {HardwareType::GR_KL300, {0x0D2C, 2, 2}},
      {HardwareType::GR_HKL330, {0x0D4A, 2, 2}},   {HardwareType::BL_KM200, {0x00C8, 2, 3}},
      {HardwareType::BL_KM300, {0x012C, 2, 3}},    {HardwareType::BL_SDS200, {0x18C8, 2, 3}},
      {HardwareType::BL_UMV200, {0x2cc8, 2, 3}},   {HardwareType::RT_TKM200, {0x04C8, 1, 6}},
      {HardwareType::RT_SDM200, {0x08C8, 1, 6}},   {HardwareType::SW_TKM200, {0x04C8, 1, 8}},
      {HardwareType::SW_TKM210, {0x04D2, 1, 8}},   {HardwareType::SW_AKM200, {0x20C8, 2, 8}},
      {HardwareType::SW_AKM210, {0x20D2, 2, 8}},   {HardwareType::SW_AKM220, {0x20DC, 2, 8}},
      {HardwareType::SW_KL200, {0x0CC8, 1, 8}},    {HardwareType::SW_ZWS200, {0x14C8, 1, 8}},
      {HardwareType::SW_ZWS200F, {0x14C9, 1, 8}},  {HardwareType::SW_ZWS200EF, {0x14CA, 2, 8}},
      {HardwareType::SW_SSL200JS, {0x0CD5, 2, 8}}, {HardwareType::SW_SSL200FS, {0x0CD6, 2, 8}},
      {HardwareType::SW_SDS200CW, {0x18C8, 2, 8}}, {HardwareType::SW_SDS200CS, {0x18C9, 2, 8}},
      {HardwareType::SW_SDS220CT, {0x18DC, 2, 8}}, {HardwareType::SW_TNY200, {0x28C8, 2, 8}},
      {HardwareType::SW_UMR200, {0x30C8, 2, 8}},   {HardwareType::SW_SK200, {0x1CC8, 2, 8}},
      {HardwareType::SW_SK201, {0x1CC9, 2, 8}},    {HardwareType::SW_SK202, {0x1CCA, 2, 8}},
      {HardwareType::SW_SKM300, {0x392c, 2, 8}},   {HardwareType::BL_SK204, {0x1CCC, 2, 3}},
      {HardwareType::SW_ZWS205, {0x14CD, 2, 8}},
  };

  auto it = partInfoTable.find(hardwareType);
  DS_REQUIRE(it != partInfoTable.end(), "Unkwnown", hardwareType);
  return it->second;
};

auto toVendorId = [](const HardwareType& hardwareType) -> uint16_t { return toPartInfo(hardwareType).vendorId; };

uint16_t toFunctionId(HardwareType hardwareType) {
  // values are manually retrieved by inspecting physical devices
  static const boost::container::flat_map<HardwareType, uint16_t> functionIdTable = {

      {HardwareType::BL_KM200,
       functionID105(DeviceClasses_t::DEVICE_CLASS_BL, FID105::extraGroups, FID105::output, FID105keys::_1)},
      {HardwareType::BL_KM300, functionID105v2(DeviceClasses_t::DEVICE_CLASS_BL, FID105::output, FID105keys::_0)},
      {HardwareType::BL_SDS200, functionID105(DeviceClasses_t::DEVICE_CLASS_BL, FID105keys::_2)},

      {HardwareType::BL_SK204,
       functionID105v2(DeviceClasses_t::DEVICE_CLASS_BL, FID105::extraEquipment, FID105keys::_1)},

      {HardwareType::GE_KM200, functionID105(DeviceClasses_t::DEVICE_CLASS_GE, FID105::output, FID105keys::_1)},

      {HardwareType::GE_KM300, functionID105(DeviceClasses_t::DEVICE_CLASS_GE, FID105::output, FID105keys::_0)},

      {HardwareType::GE_TKM300,
       functionID105(DeviceClasses_t::DEVICE_CLASS_GE, FID105::extraEquipment, FID105::output, FID105keys::_1)},

      {HardwareType::GE_KL200, functionID105(DeviceClasses_t::DEVICE_CLASS_GE, FID105::output, FID105keys::_1)},

      {HardwareType::GE_TKM210, functionID105(DeviceClasses_t::DEVICE_CLASS_GE, FID105::output, FID105keys::_1)},
      {HardwareType::GE_TKM230, functionID105(DeviceClasses_t::DEVICE_CLASS_GE, FID105keys::_2)},

      {HardwareType::GE_TKM220, functionID105(DeviceClasses_t::DEVICE_CLASS_GE, FID105keys::_1)},
      {HardwareType::GE_SDM200JS, functionID105(DeviceClasses_t::DEVICE_CLASS_GE, FID105::output, FID105keys::_1)},

      {HardwareType::GE_SDM200CS, functionID105(DeviceClasses_t::DEVICE_CLASS_GE, FID105::output, FID105keys::_1)},

      {HardwareType::GE_SDM200, functionID105(DeviceClasses_t::DEVICE_CLASS_GE, FID105::output, FID105keys::_1)},

      {HardwareType::GE_SDS200CW, functionID105(DeviceClasses_t::DEVICE_CLASS_GE, FID105::output, FID105keys::_2)},

      {HardwareType::GE_SDS200CS, functionID105(DeviceClasses_t::DEVICE_CLASS_GE, FID105::output, FID105keys::_2)},

      {HardwareType::GE_SDS220CT, functionID105(DeviceClasses_t::DEVICE_CLASS_GE, FID105::output, FID105keys::_2)},

      {HardwareType::GE_ZWD300W, functionID105v2(DeviceClasses_t::DEVICE_CLASS_GE, FID105::output, FID105keys::_0)},

      {HardwareType::GE_SDM300W,
       functionID105v2(DeviceClasses_t::DEVICE_CLASS_GE, FID105::extraEquipment, FID105::output, FID105keys::_1)},

      {HardwareType::GE_SDM300S,
       functionID105v2(DeviceClasses_t::DEVICE_CLASS_GE, FID105::extraEquipment, FID105::output, FID105keys::_1)},

      {HardwareType::GR_KL200,
       functionID105(DeviceClasses_t::DEVICE_CLASS_GR, FID105::varRampTime, FID105::output, FID105keys::_1)},

      {HardwareType::GR_KL210,
       functionID105(DeviceClasses_t::DEVICE_CLASS_GR, FID105::varRampTime, FID105::output, FID105keys::_1)},

      {HardwareType::GR_KL220,
       functionID105(DeviceClasses_t::DEVICE_CLASS_GR, FID105::varRampTime, FID105::output, FID105keys::_1)},

      {HardwareType::GR_HKL230,
       functionID105(DeviceClasses_t::DEVICE_CLASS_GR, FID105::varRampTime, FID105::output, FID105keys::_0)},

      {HardwareType::GR_TKM200, functionID105(DeviceClasses_t::DEVICE_CLASS_GR, FID105keys::_4)},
      {HardwareType::GR_TKM210, functionID105(DeviceClasses_t::DEVICE_CLASS_GR, FID105keys::_1)},

      {HardwareType::GN_TKM200, functionID105(DeviceClasses_t::DEVICE_CLASS_GN, FID105keys::_4)},
      {HardwareType::GN_TKM210, functionID105(DeviceClasses_t::DEVICE_CLASS_GN, FID105keys::_1)},

      {HardwareType::GN_KM200, functionID100(DeviceClasses_t::DEVICE_CLASS_GN, FID100::output, FID100keys::_1)},

      {HardwareType::RT_TKM200, functionID100(DeviceClasses_t::DEVICE_CLASS_RT, FID100keys::_2)},

      {HardwareType::RT_SDM200, functionID100(DeviceClasses_t::DEVICE_CLASS_RT, FID100keys::_2)},

      {HardwareType::GR_HKL330,
       functionID105v2(DeviceClasses_t::DEVICE_CLASS_GR, FID105::varRampTime, FID105::output, FID105keys::_0)},

      {HardwareType::SW_AKM200,
       functionID105(DeviceClasses_t::DEVICE_CLASS_SW, FID105::extraEquipment, FID105keys::_4)},

      {HardwareType::SW_AKM210,
       functionID105(DeviceClasses_t::DEVICE_CLASS_SW, FID105::extraEquipment, FID105keys::_2)},

      {HardwareType::SW_KL200, functionID105(DeviceClasses_t::DEVICE_CLASS_SW, FID105::output, FID105keys::_1)},

      {HardwareType::SW_SSL200JS, functionID105(DeviceClasses_t::DEVICE_CLASS_SW, FID105::output, FID105keys::_0)},

      {HardwareType::SW_SSL200FS, functionID105(DeviceClasses_t::DEVICE_CLASS_SW, FID105::output, FID105keys::_0)},

      {HardwareType::SW_SK200, functionID105(DeviceClasses_t::DEVICE_CLASS_SW, FID105::extraEquipment, FID105keys::_0)},
      {HardwareType::SW_SK201, functionID105(DeviceClasses_t::DEVICE_CLASS_SW, FID105::extraEquipment, FID105keys::_0)},

      {HardwareType::SW_SK202, functionID105(DeviceClasses_t::DEVICE_CLASS_SW, FID105::extraEquipment, FID105keys::_0)},

      {HardwareType::SW_TKM200, functionID105(DeviceClasses_t::DEVICE_CLASS_SW, FID105keys::_4)},

      {HardwareType::SW_TKM210, functionID105(DeviceClasses_t::DEVICE_CLASS_SW, FID105keys::_2)},

      {HardwareType::SW_SDS200CW, functionID105(DeviceClasses_t::DEVICE_CLASS_SW, FID105keys::_2)},

      {HardwareType::SW_SDS200CS, functionID105(DeviceClasses_t::DEVICE_CLASS_SW, FID105keys::_2)},

      {HardwareType::SW_SDS220CT, functionID105(DeviceClasses_t::DEVICE_CLASS_SW, FID105keys::_2)},

      {HardwareType::SW_ZWS200, functionID105(DeviceClasses_t::DEVICE_CLASS_SW, FID105::output, FID105keys::_1)},

      {HardwareType::SW_ZWS200F, functionID105(DeviceClasses_t::DEVICE_CLASS_SW, FID105::output, FID105keys::_1)},

      {HardwareType::SW_ZWS200EF, functionID105(DeviceClasses_t::DEVICE_CLASS_SW, FID105::output, FID105keys::_1)},

      {HardwareType::SW_ZWS205, functionID105(DeviceClasses_t::DEVICE_CLASS_SW, FID105::output, FID105keys::_1)},

      {HardwareType::SW_SKM300,
       functionID105v2(DeviceClasses_t::DEVICE_CLASS_SW, FID105::extraEquipment, FID105keys::_4)},
  };

  auto it = functionIdTable.find(hardwareType);
  DS_REQUIRE(it != functionIdTable.end(), "Unkwnown", hardwareType);
  return it->second;
}

Dsuid createNextDSUID() {
  // TODO scoped_lock if tests are not sequential
  // 00 00 00 00 00 00                              |
  //                  00 00 00 00                   |
  //                             07 57              |
  //                                  57 50 00 00   | serial number
  //                                             03 | vdc multi-device
  static Dsuid seed{"0000000000000000000007575750000003"};
  dsuid_get_next_dsuid(&seed, &seed);
  return seed;
}

// fw version in use
// https://confluence.digitalstrom.org/pages/viewpage.action?pageId=549388313
uint16_t revisionIdMock(const DeviceMockConf& conf) {
  return conf.revisionId.value_or(0x356); // random pick
}

bool isValidMainDsuid(const dsuid_t& dsuid, int deviceCount) {
  uint32_t serial;
  // -> serial is byte 12-16 in dsuid
  // mod(serial, deviceCount) -> multiDeviceIndex
  dsuid_get_serial_number(&dsuid, &serial);
  return ((serial % deviceCount) == 0); // serial+1 -> sub-device-1, etc
}

dsuid_t createMultiDsuid(int deviceCount) {
  auto dsuid = createNextDSUID();
  while (!isValidMainDsuid(dsuid, deviceCount)) {
    dsuid = createNextDSUID();
  }
  return dsuid;
}

std::vector<std::shared_ptr<Device>> createSWSKM300(Apartment& apartment, const DeviceMockConf& conf) {
  std::vector<std::shared_ptr<Device>> devices;

  // bundle of 4 tb with 4 consecutive dsuid, each has one input,
  auto dsuid = createMultiDsuid(4);

  REQUIRE_CALL(dsm_api_mock, DsmApiGetDeviceDescription(_, _, _, _, _, _, _)).TIMES(4).RETURN(0);

  for (int i = 0; i < 4; i++) {
    devices.push_back(apartment.allocateDevice(dsuid));
    devices.back()->setPartiallyFromSpec(
        DeviceSpec{.FunctionID = toFunctionId(HardwareType::SW_SKM300),
                   .ProductID = DEVICE_TYPE_SKM << 10 | 300,
                   .VendorID = toVendorId(HardwareType::SW_SKM300),
                   .revisionId = revisionIdMock(conf),
                   .activeGroup = static_cast<int>(ApplicationType::lights),
                   .defaultGroup = static_cast<int>(ApplicationType::lights)}); // exception from the rule
    DS_REQUIRE(devices.back()->multiDeviceIndex() == i);
    dsuid = createNextDSUID();
  }
  return devices;
}

std::vector<std::shared_ptr<Device>> createSWUMR200(Apartment& apartment, const DeviceMockConf& conf) {
  std::vector<std::shared_ptr<Device>> devices;

  // bundle of 4 tb with 4 consecutive dsuid
  auto dsuid = createMultiDsuid(4);

  auto partInfo = toPartInfo(conf.hardwareType);

  REQUIRE_CALL(dsm_api_mock, DsmApiGetDeviceDescription(_, _, _, _, _, _, _)).TIMES(4).RETURN(0);
  // 2 t|akm  2 inputs, configure as binary/taster + 2 relais output
  // reliable transport, in the back channel != regular TKM/AKM

  auto functionId = [](int index) -> uint16_t {
    switch (index) {
      case 0:
      case 1: return functionID105v2(DeviceClasses_t::DEVICE_CLASS_SW, FID105::extraEquipment, FID105keys::_2);
      case 2:
        return functionID105v2(DeviceClasses_t::DEVICE_CLASS_SW, FID105::output, FID105::extraEquipment,
                               FID105keys::_0);
      case 3: return functionID105v2(DeviceClasses_t::DEVICE_CLASS_SW, FID105::output, FID105keys::_0);
    }
    DS_FAIL_REQUIRE(index);
  };

  for (int i = 0; i < 4; i++) {
    devices.push_back(
        apartment.allocateDevice([i, &dsuid]() -> dsuid_t { return (i == 0) ? dsuid : createNextDSUID(); }()));
    devices.back()->setPartiallyFromSpec(DeviceSpec{.FunctionID = functionId(i),
                                                    .ProductID = DEVICE_TYPE_UMR << 10 | 200,
                                                    .VendorID = toVendorId(conf.hardwareType),
                                                    .revisionId = revisionIdMock(conf),
                                                    .activeGroup = partInfo.color,
                                                    .defaultGroup = partInfo.color}); // assumption

    DS_REQUIRE(devices.back()->multiDeviceIndex() == i);
  }

  return devices;
}

std::vector<std::shared_ptr<Device>> createGRKL300(Apartment& apartment, const DeviceMockConf& conf) {
  std::vector<std::shared_ptr<Device>> devices;
  DS_ASSERT(conf.hardwareType == HardwareType::GR_KL300);

  auto dsuid = createMultiDsuid(2);

  auto groups = [&groups = conf.groups] {
    if (groups.empty()) {
      return std::vector{GroupIDGray};
    } else {
      return groups;
    }
  }();

  REQUIRE_CALL(dsm_api_mock, DsmApiGetDeviceDescription(_, _, _, _, _, _, _)).TIMES(2).RETURN(0);
  // 2 dsuids - 1 input 1 output + 1 input

  devices.push_back(apartment.allocateDevice(dsuid));
  setupBaseDevice(devices.back(),
                  /* 1 output, 1 input */
                  DeviceSpec{.FunctionID = functionID105v2(DeviceClasses_t::DEVICE_CLASS_GR, FID105::varRampTime,
                                                           FID105::output, FID105keys::_1),
                             .ProductID = DEVICE_TYPE_KL << 10 | 300,
                             .VendorID = toVendorId(conf.hardwareType),
                             .revisionId = revisionIdMock(conf),
                             .LTMode = conf.buttonInputMode.value_or(ButtonInputMode::TWO_WAY_DW_WITH_INPUT2),
                             .Groups = groups,
                             .buttonID = ButtonId_Area1,
                             .buttonActiveGroup = GroupIDGray,
                             .activeGroup = static_cast<int>(ApplicationType::blinds),
                             .defaultGroup = static_cast<int>(ApplicationType::blinds)});

  DS_REQUIRE(devices.back()->multiDeviceIndex() == 0);
  devices.push_back(apartment.allocateDevice(createNextDSUID()));
  setupBaseDevice(
      devices.back(),
      /* 1 input/binary input */
      DeviceSpec{
          .FunctionID = functionID105v2(DeviceClasses_t::DEVICE_CLASS_GR, FID105::extraEquipment, FID105keys::_1),
          .ProductID = DEVICE_TYPE_KL << 10 | 300,
          .VendorID = toVendorId(conf.hardwareType),
          .revisionId = revisionIdMock(conf),
          .LTMode = conf.buttonInputMode.value_or(ButtonInputMode::TWO_WAY_UP_WITH_INPUT1),
          .Groups = groups,
          .buttonID = ButtonId_Area1,
          .buttonActiveGroup = GroupIDGray,
          .binaryInputs = std::vector<DeviceBinaryInputSpec_t>{{.TargetGroup = GroupIDBlack,
                                                                .InputType = BinaryInputType::DoorContact,
                                                                .InputID = BinaryInputId::APP_MODE,
                                                                .Independent = false}},
          .binaryInputsValid = true,
          .activeGroup = static_cast<int>(ApplicationType::blinds),
          .defaultGroup = static_cast<int>(ApplicationType::blinds)});

  DS_REQUIRE(devices.back()->multiDeviceIndex() == 1);
  return devices;
}

std::vector<std::shared_ptr<Device>> createUMV(Apartment& apartment, const DeviceMockConf& conf,
                                                 uint16_t productNumber, dss::DeviceClasses_t color) {
  std::vector<std::shared_ptr<Device>> devices;

  // bundle of 4 tb with 4 consecutive dsuid,first dsuid have input and output othes has output only,
  auto dsuid = createMultiDsuid(4);
  auto partInfo = toPartInfo(conf.hardwareType);

  devices.push_back(apartment.allocateDevice(dsuid));
  devices.back()->setPartiallyFromSpec(DeviceSpec{.FunctionID = functionID105(color, FID105::output, FID105keys::_1),
                                                  .ProductID = productNumber,
                                                  .VendorID = toVendorId(conf.hardwareType),
                                                  .revisionId = revisionIdMock(conf),
                                                  .activeGroup = partInfo.color,
                                                  .defaultGroup = partInfo.color}); // assumption
  DS_REQUIRE(devices.back()->multiDeviceIndex() == 0);

  for (int i = 1; i < 4; i++) {
    dsuid = createNextDSUID();
    devices.push_back(apartment.allocateDevice(dsuid));
    devices.back()->setPartiallyFromSpec(DeviceSpec{.FunctionID = functionID105(color, FID105::output, FID105keys::_0),
                                                    .ProductID = productNumber,
                                                    .VendorID = toVendorId(conf.hardwareType),
                                                    .revisionId = revisionIdMock(conf),
                                                    .activeGroup = partInfo.color,
                                                    .defaultGroup = partInfo.color}); // assumption
    DS_REQUIRE(devices.back()->multiDeviceIndex() == i);
  }
  return devices;
}

std::shared_ptr<Device> createDevice(Apartment& apartment, const DeviceMockConf& conf) {
  auto dev = apartment.allocateDevice(createNextDSUID());

  // setPartiallyFromSpec triggers lookup in productlist_dsd.csv
  REQUIRE_CALL(dsm_api_mock, DsmApiGetDeviceDescription(_, _, _, _, _, _, _)).TIMES(0, 4).RETURN(0);

  auto vendorId = [partInfo = toPartInfo(conf.hardwareType)]() -> uint16_t { return partInfo.vendorId; };
  auto productId = [partInfo = toPartInfo(conf.hardwareType)]() { return partInfo.productId; };

  auto partInfo = toPartInfo(conf.hardwareType);

  switch (conf.hardwareType) {
    case HardwareType::BL_KM200:
    case HardwareType::BL_KM300:
    case HardwareType::BL_SDS200:
    case HardwareType::BL_SK204:
    case HardwareType::GE_KM200:
    case HardwareType::GE_KM300:
    case HardwareType::GE_TKM300:
    case HardwareType::GE_KL200:
    case HardwareType::GE_TKM210:
    case HardwareType::GE_TKM230:
    case HardwareType::GE_TKM220:
    case HardwareType::GE_SDM200JS:
    case HardwareType::GE_SDM200CS:
    case HardwareType::GE_SDM200:
    case HardwareType::GE_SDS200CW:
    case HardwareType::GE_SDS200CS:
    case HardwareType::GE_SDS220CT: // missing unit test
    case HardwareType::GE_ZWD300W:
    case HardwareType::GE_SDM300W:
    case HardwareType::GE_SDM300S:
    case HardwareType::GR_KL200:
    case HardwareType::GR_KL210:
    case HardwareType::GR_KL220:
    case HardwareType::GR_HKL230:
    case HardwareType::GR_TKM200:
    case HardwareType::GR_TKM210:
    case HardwareType::GN_TKM200:
    case HardwareType::GN_TKM210:
    case HardwareType::GN_KM200:
    case HardwareType::RT_TKM200:
    case HardwareType::RT_SDM200:
    case HardwareType::GR_HKL330:
    case HardwareType::SW_KL200:
    case HardwareType::SW_SSL200JS:
    case HardwareType::SW_SSL200FS:
    case HardwareType::SW_SK200:
    case HardwareType::SW_SK201:
    case HardwareType::SW_SK202:
    case HardwareType::SW_TKM200:
    case HardwareType::SW_TKM210:
    case HardwareType::SW_SDS200CW:
    case HardwareType::SW_SDS200CS:
    case HardwareType::SW_SDS220CT:
    case HardwareType::SW_ZWS200:
    case HardwareType::SW_ZWS200F:
    case HardwareType::SW_ZWS200EF:
    case HardwareType::SW_ZWS205:
      dev->setPartiallyFromSpec(DeviceSpec{.FunctionID = toFunctionId(conf.hardwareType),
                                           .ProductID = productId(),
                                           .VendorID = vendorId(),
                                           .revisionId = revisionIdMock(conf),
                                           .activeGroup = partInfo.color,
                                           .defaultGroup = partInfo.color}); // assumption
      break;

    case HardwareType::SW_AKM200: // 4 inputs, sensor only
    case HardwareType::SW_AKM210: // 2 inputs, sensor only
      dev->setPartiallyFromSpec(DeviceSpec{.FunctionID = toFunctionId(conf.hardwareType),
                                           .ProductID = productId(),
                                           .VendorID = vendorId(),
                                           .revisionId = revisionIdMock(conf),
                                           .activeGroup = partInfo.color,
                                           .defaultGroup = partInfo.color}); // assumption
      dev->setButtonInputMode(ButtonInputMode::AKM_STANDARD);
      break;

    case HardwareType::GE_UMV200:
      return createUMV(apartment, conf, DEVICE_TYPE_UMV << 10 | 200, dss::DeviceClasses_t::DEVICE_CLASS_GE).front();

    case HardwareType::BL_UMV200:
      return createUMV(apartment, conf, DEVICE_TYPE_UMV << 10 | 200, dss::DeviceClasses_t::DEVICE_CLASS_BL).front();

    case HardwareType::SW_UMR200: return createSWUMR200(apartment, conf).front();
    case HardwareType::GR_KL300: return createGRKL300(apartment, conf).front();
    case HardwareType::SW_SKM300: return createSWSKM300(apartment, conf).front();

    case HardwareType::GE_UMV210:                  // Not implemented
    case HardwareType::SW_TNY200: [[fallthrough]]; // Not implemented
    case HardwareType::SW_AKM220: [[fallthrough]]; // Never implemented
    case HardwareType::unknown: [[fallthrough]];
    default: DS_FAIL_REQUIRE("No mock device for", conf.hardwareType, "available");
  }
  return dev;
};

} // namespace dss
