/*
    Copyright (c) 2020 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#include "physicalmodelitem.h"

ds::LogChannel dsLogChannel("modelPhysicalModelItem");

namespace dss {

PhysicalModelItem::PhysicalModelItem() = default;
PhysicalModelItem::~PhysicalModelItem() = default;

bool PhysicalModelItem::isPresent() const {
  return m_isPresent & m_isConnected;
}

void PhysicalModelItem::setIsPresent(bool isPresent) {
  DS_DEBUG_ENTER(isPresent, m_isPresent, m_inactiveSince);
  bool wasPresent = m_isPresent;

  m_isPresent = isPresent;
  if (m_isPresent) {
    m_inactiveSince = DateTime::NullDate;
  } else {
    if (wasPresent || (m_inactiveSince == DateTime::NullDate)) {
      m_inactiveSince = DateTime();
    }
  }
  DS_DEBUG_LEAVE(m_isPresent, m_inactiveSince);
}

void PhysicalModelItem::setInactiveSince(DateTime inactiveSince) {
  DS_DEBUG_ENTER(inactiveSince, m_inactiveSince, m_isPresent);
  m_inactiveSince = inactiveSince;
}

void PhysicalModelItem::setIsConnected(bool isConnected) {
  m_isConnected = isConnected;
}

} // namespace dss
