#include "powerline.h"

#include <ds/enum.h>
#include <ds/log.h>

static ds::LogChannel dsLogChannel("modelPowerline");

DS_REFLECTED_ENUM_WITH_INITIALIZERS((dss::model), SubClass,
                                    ((R100)(0b000))((R105)(0b100))((R105v2Heating)(0b1000))((R105v2Climate)(0b1001)));

namespace dss::model {

int extractClass(uint16_t functionId) {
  return (functionId >> 12) & 0xf;
}

SubClass extractSubclass(uint16_t functionId) {
  return static_cast<SubClass>((functionId >> 6) & 0x3f);
}

uint8_t decodeInputCount(uint16_t functionId) {
  uint8_t inputCount = 0;
  if ((functionId & 0xffc0) == 0x1000) {
    // light class + dS-standard R100
    switch (functionId & 0x7) {
      case 0: inputCount = 1; break;
      case 1: inputCount = 2; break;
      case 2: inputCount = 4; break;
      case 7: inputCount = 0; break;
    }
  } else if (((functionId & 0x0fc0) == 0x0100) || ((functionId & 0x0fc0) == 0x01c0) ||
             ((functionId & 0x0fc0) == 0x0200) || ((functionId & 0x0fc0) == 0x0240)) {
    // subclass 4: dS-standard R105
    // subclass 7: dS-standard R105 / with group membership assignment v2
    // subclass 8: clima only: R105 with internal temperature regulation and group bytes v2
    // subclass 9: clima only: R105 with internal temperature regulation, active cooling, group bytes v2
    switch (functionId & 0x3) {
      case 0: inputCount = 0; break;
      case 1: inputCount = 1; break;
      case 2: inputCount = 2; break;
      case 3: inputCount = 4; break;
    }
  } else {
    // ApartmentStructurePoller starts before readout is complete, at that time
    // FunctionId is not yet fetched from device nor part of the apartment.xml
    // see isInDsModel
    DS_DEBUG("unknown functionId format", functionId, "assume no inputs available");
    if (functionId != 0) {
      auto deviceSubclass = extractSubclass(functionId);
      auto deviceClass = extractClass(functionId);
      DS_WARNING("unknown functionId format", functionId, deviceClass, deviceSubclass, "assume no inputs available");
    }
    inputCount = 0;
  }
  return inputCount;
}
} // namespace dss::model
