#!/bin/sh

###
## Generate a build info header; unix version
#
# Johannes Winkelmann, johannes.winkelmann@aizo.com

set -e

BASEDIR=$1
TARGETDIR=$2
BUILD_INFO_TMP=build_info.h.tmp.$(date +%s)
BUILD_INFO=build_info.h

if [ -z "$TARGETDIR" ] ; then
  echo "Usage: $(basename $0) <base directory> <build_info target_dir>"
  exit 1
fi

rcs_rev="-"
if [ -e $BASEDIR/.git ]; then
  rcs_rev="$(git --git-dir=$BASEDIR/.git describe --tags --dirty)"
else
  echo "Failed to determine RCS revision in $BASEDIR"
  exit 1
fi


echo "/* $BUILD_INFO generated by $(basename $0)        */" \
	> $TARGETDIR/$BUILD_INFO_TMP
echo "/* do not edit, modifications will be overwritten */" \
	>> $TARGETDIR/$BUILD_INFO_TMP


echo "#define DSS_RCS_REVISION \"$rcs_rev\""       >> $TARGETDIR/$BUILD_INFO_TMP
echo "#define DSS_BUILD_USER   \"$(whoami)\""      >> $TARGETDIR/$BUILD_INFO_TMP
echo "#define DSS_BUILD_HOST   \"$(hostname)\""    >> $TARGETDIR/$BUILD_INFO_TMP

# check whether the actual contents changed; don't overwrite otherwise
if [ -f $TARGETDIR/$BUILD_INFO ]; then
    if [ -z "$(diff -q $TARGETDIR/$BUILD_INFO_TMP $TARGETDIR/$BUILD_INFO)" ]; then
	echo "Info: No changes since last $BUILD_INFO generation"
	rm $TARGETDIR/$BUILD_INFO_TMP
	exit 0
    fi
fi    
mv $TARGETDIR/$BUILD_INFO_TMP $TARGETDIR/$BUILD_INFO
